/** Utilities for Jupyterlab Bookmarks plugin.
 * Copyright (c) 2020, Dr. Nandor Poka, All rights reserved.
 */
import { ISettingRegistry } from '@jupyterlab/settingregistry';
import { IDisposable } from '@lumino/disposable';
import { Menu } from '@lumino/widgets';
import { CommandRegistry } from '@lumino/commands';
import { ILauncher } from '@jupyterlab/launcher';
import { INotebookTracker, NotebookPanel, INotebookModel } from '@jupyterlab/notebook';
import { DocumentRegistry } from '@jupyterlab/docregistry';
export declare const commandPrefix = "jupyterlab-bookmarks-extension:";
export declare const VERSION = "0.5.1";
export declare const TITLE_PLAIN = "Bookmarks";
export declare const TITLE: string;
export declare const DISABLED_TITLE: string;
export declare const NOTEBOOK_FACTORY = "Notebook";
export declare const bookmarkCommands: Map<string, IDisposable>;
export declare const bookmarkLaunchers: Map<string, IDisposable>;
export declare const bookmarkMenuItems: Map<string, Menu.IItem>;
/**
 * Load the settings for this extension
 *
 * @param settings Extension settings
 */
export declare function loadSetting(settings: ISettingRegistry.ISettings): void;
export declare function setBookmarks(incomingBookmarks: Array<Array<string>>): void;
export declare function getBookmarks(): Array<Array<string>>;
export declare function getSettingsObject(): ISettingRegistry.ISettings;
export declare function setSettingsObject(incomingSettingsObject: ISettingRegistry.ISettings): void;
/** Adds a `bookmarkItem`, to the `commands` list, to the Launcher and to the Bookmarks menu.
 * @async
 * @param commands - A `CommandRegistry` instance that holds the commands registered in the JL app.
 * @param launcher - The `Launcher` instance to add the bookmark to.
 * @param bookmarkItem - `string[]`. The bookmarkItem to be added.
 * @param skipDuplicateCheck - `boolean`. Optional parameter to skip or not to skip
 * duplicate check.
 * @returns `Promise<boolean>`
 */
export declare function addBookmark(commands: CommandRegistry, launcher: ILauncher, bookmarkItem: string[], skipDuplicateCheck?: boolean): Promise<boolean>;
/**
 * Updates the JupyterLab apps `commands` variable with the command that is assigned to launch the bookmark.
 * @param commands - The `CommandRegistry` instance to add the command to.
 * @param bookmarkItem - `string[]` that has the information about the bookmark.
 * @returns `void`
 */
export declare function updateCommands(commands: CommandRegistry, bookmarkItem: string[]): void;
/**
 * Updates the `Launcher` with the bookmark.
 * @param launcher - The `Launcher` instance to update.
 * @param bookmarkItem - string[] that holds the bookmark to add.
 * @returns `void`
 */
export declare function updateLauncher(launcher: ILauncher, bookmarkItem: string[]): void;
/**
 * Updates the plugin's menu with the bookmark.
 * @param bookmarkItem - string[] that holds the bookmark to add.
 * @returns `void`
 */
export declare function updateMenu(bookmarkItem: string[]): void;
/**
 * Deletes the bookmark, and updates the `Launcher` the `Menu` and the `commands`.
 * @async
 * @param bookmarkToDelete - `string[]` the bookmark to delete.
 * @returns `Promise<void>
 */
export declare function deleteBookmark(bookmarkToDelete: string): Promise<void>;
/**
 * Adds a bookmark item from the context menu of an open notebook.
 * @async
 * @param commands - A `CommandRegistry` instance that holds the commands registered in the JL app.
 * @param launcher - The `Launcher` instance to add the bookmark to.
 * @param currentDocName - `string` the name of the open notebbook
 * @param currentDocPath - `string` the path of the open notebook
 * @returns `Promise<void>`
 */
export declare function addBookmarkItem(commands: CommandRegistry, launcher: ILauncher, currentDocName: string, currentDocPath: string): Promise<void>;
/**
 * Update the plugin's settings with the bookmark.
 * @async
 * @param bookmarkItem - `string[]` that stores the bookmark to be saved to the settings.
 * @returns `Promise<void>`
 */
export declare function updateSettings(bookmarkItem: string[]): Promise<void>;
/**
 * Synchronises a notebook that has been opened from a temporary location back to it's original instance when the file is saved.
 * Makes a HTTP call to the plugins server extension that does the actual copying.
 * @param bookmarkedNotebookModel - `DocumentRegistry.IContext<INotebookModel>` the context object for the current NotebookWdiget.
 */
export declare function syncBookmark(bookmarkedNotebookModel: DocumentRegistry.IContext<INotebookModel>): void;
/**
 * Sets up the autosyncing for a notebook, by connecting the `syncBookmark` function to the notebook's context's fileChanged signal.
 * @param notebookPanel - `NotebookPanel`, the widget hosting the notebook.
 * @param notebookTracker - `INotebookTracker` instance as the sender
 */
export declare function addAutoSyncToBookmark(notebookTracker: INotebookTracker, notebookPanel: NotebookPanel): void;
